#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'CountEncoder'
__author__ = 'JieYuan'
__mtime__ = '19-1-10'
"""
import pandas as pd
from ...pipe import tqdm

from concurrent.futures import ProcessPoolExecutor
from sklearn.base import BaseEstimator, TransformerMixin


class CountEncoder(BaseEstimator, TransformerMixin):

    def __init__(self, normalize=False, dropna=False):
        self.dropna = dropna
        self.normalize = normalize
        self.counter_ls = []

    def fit(self, df: pd.DataFrame):
        self.w = df.shape[1]
        self.counter_ls = list(self._pe(self._fit, tqdm(self._get_series_ls(df), 'Fitting ...'), self.w))
        return self

    def transform(self, df):
        assert self.w == df.shape[1]
        _ = self._pe(self._transform, tqdm(zip(self.counter_ls, self._get_series_ls(df)), 'Transforming ...'), self.w)
        return pd.concat(_, 1)

    # def fit_transform(self, df):
    #     return self.fit(df).transform(df)

    def _fit(self, s: pd.Series):
        _ = (s.value_counts(normalize=self.normalize, dropna=self.dropna)
             .reset_index(name='count_' + s.name))
        return _

    def _transform(self, args):
        c, s = args
        _ = (s.to_frame('index')
             .merge(c, 'left')
             .drop('index', 1)
             .fillna(0))  # 不在训练集的补0
        return _

    def _pe(self, func, iterable, feat_num):
        with ProcessPoolExecutor(5 if feat_num > 5 else feat_num) as pool:
            return pool.map(func, iterable)

    def _get_series_ls(self, df):
        return [v for k, v in df.items()]


if __name__ == '__main__':
    df = pd.DataFrame({'a': ['1', '2'], 'b': ['2', '2']})
    c = CountEncoder()
    c.fit(df)
    print(c)
    print(c.transform(df))
    print(c.fit_transform(df))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'fire'
__author__ = 'JieYuan'
__mtime__ = '19-1-10'
"""

import fire


class Config(object):
    x = 1
    y = 1


opt = Config()


class G:
    @staticmethod
    def gen(**kwargs):
        for i in kwargs.items():
            setattr(opt, *i)

        return opt.x + opt.y


gen = G.gen
if __name__ == '__main__':
    fire.Fire()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-3-12'
"""

import pandas as pd

t = pd.Timestamp(pd.datetime.today())


class TimeFeats(object):
    period_dict = dict(
        second=1,
        minute=2,
        hour=3,
        day=4,
        week=5,
        month=6,
        quarter=7,
        year=8
    )

    def __init__(self, max_time_period='year'):
        self.period = self.period_dict.get(max_time_period, 0)

    def feat_base(self, t=pd.Timestamp(pd.datetime.today())):
        ts = []
        if self.period_dict['year'] <= self.period:
            ts.append(t.is_leap_year)
            ts.append(t.year)
            ts.append(t.weekofyear)
            ts.append(t.dayofyear)
        elif self.period_dict['quarter'] <= self.period:
            ts.append(t.quarter)
            ts.append(t.quarter)
            ts.append(t.quarter)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'demo_flask_restful'
__author__ = 'JieYuan'
__mtime__ = '19-3-1'
"""

from flask import Flask
from flask_restful import Api, Resource, reqparse, abort, request

parser = reqparse.RequestParser()
parser.add_argument('args')  # 匹配需要的key


class Post(Resource):
    routing = '/post'

    def __init__(self):
        pass

    def get(self):
        dic = parser.parse_args()  # **dic
        return dic

    def post(self):
        """str(i, encoding='utf-8'): 字节转字符串
        application/json: request.form
        >> requests.post(url, data=[('key', 'value')])
        application/x-www-form-urlencoded: request.json
        >> requests.post(url, json={'key': 'value'})
        request.files: request.files['a']
        >> requests.post(url, files={'key':  open('./xx.txt')})
        """
        dic = request.json  # request.form
        return dic


if __name__ == '__main__':
    app = Flask(__name__)
    api = Api(app)
    api.add_resource(Post, Post.routing)
    app.run(debug=True)



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'rfe'
__author__ = 'JieYuan'
__mtime__ = '19-1-30'
"""

"""
https://www.kaggle.com/roydatascience/recursive-feature-selection-new-transactions-elo
"""


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'sanic'
__author__ = 'JieYuan'
__mtime__ = '19-3-1'
"""

from sanic import Sanic
from sanic.response import json

app = Sanic()


@app.route('/get')
async def get(request):
    return json(request.args)


@app.route('/post', methods=['POST'])
async def post(request):
    return json(request.json)


# def test():
#     import requests
#     r = requests.get('http://0.0.0.0:8000/get', params={'a': 1})
#     r.json()
#     # {'a': ['1']}
#
#     r = requests.post('http://0.0.0.0:8000/post', json={'a': 1})
#     r.json()
#     # {'a': 1}


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=8000, debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'iter'
__author__ = 'JieYuan'
__mtime__ = '18-12-14'
"""

from .utils.xx import xx
from .eda import DataFrameSummary
from .utils import cprint

try:
    from IPython import get_ipython

    if 'IPKernelApp' not in get_ipython().config:
        raise ImportError("console")
except:
    from tqdm import tqdm

else:
    from tqdm import tqdm_notebook as tqdm
#########################################################################
from sklearn.datasets import make_classification
from sklearn.model_selection import cross_val_score
from sklearn.ensemble import *
from sklearn.linear_model import *
from sklearn.model_selection import StratifiedKFold, KFold
#########################################################################
import warnings

warnings.filterwarnings("ignore")
import jieba
import json
import pickle
import numpy as np
import pandas as pd
from functools import reduce
from collections import Counter, OrderedDict
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor

from sklearn.feature_extraction.text import TfidfVectorizer

# pd.set_option('display.max_rows', 1024)
# pd.set_option('display.max_columns', 128)
# pd.set_option('max_colwidth', 128)  # 列宽
# pd.set_option('expand_frame_repr', False)  # 允许换行显示


import matplotlib.pyplot as plt

# plt.style.use('ggplot')
plt.rcParams['font.sans-serif'] = ['Simhei']  # 中文乱码的处理
plt.rcParams['font.family'] = 'sans-serif'
plt.rcParams['axes.unicode_minus'] = False  # 负号
plt.rcParams["text.usetex"] = False
plt.rcParams["legend.numpoints"] = 1
plt.rcParams["figure.figsize"] = (12, 6)  # (8, 6)
plt.rcParams["figure.dpi"] = 128
plt.rcParams["savefig.dpi"] = plt.rcParams["figure.dpi"]
plt.rcParams["font.size"] = 10
plt.rcParams["pdf.fonttype"] = 42

import seaborn as sns

sns.set(style="darkgrid")  # darkgrid, whitegrid, dark, white,和ticks

# sns.plotting_context()
# sns.axes_style()
#########################################################################
TfidfVectorizer


# 序列化
# df.to_hdf('./data.h5', 'w', complib='blosc', complevel=8)
def reader(fname='./tmp.txt', sep=',', mode='r'):
    with open(fname, mode) as f:
        for l in f:
            yield l.strip().split(sep)


@xx
def xwrite(iterable, fname, mode='w', glue='\n'):
    with open(fname, mode) as f:
        for item in iterable:
            f.write(str(item) + glue)


@xx
def xpickle_dump(obj, file='tmp.pkl'):
    with open(file, 'wb') as f:
        pickle.dump(obj, f)


@xx
def xpickle_load(file):
    with open(file, 'rb') as f:
        return pickle.load(f)


# 统计函数: 待补充groupby.agg
xsummary = xx(lambda iterable: DataFrameSummary(list(iterable) | xDataframe)['iterable'])
xvalue_counts = xx(
    lambda iterable, normalize=False, bins=None: pd.value_counts(list(iterable), normalize=normalize, bins=bins))

__funcs = [sum, min, max, abs, len, np.mean, np.median]
xsum, xmin, xmax, xabs, xlen, xmean, xmedian = [xx(i) for i in __funcs]

xnorm = xx(lambda iterable, ord=2: np.linalg.norm(list(iterable), ord))
xcount = xx(lambda iterable: Counter(list(iterable)))

xunique = xx(lambda iterable: list(OrderedDict.fromkeys(list(iterable))))  # 移除列表中的重复元素(保持有序)
xsort = xx(lambda iterable, reverse=False, key=None: sorted(list(iterable), key=key, reverse=reverse))

xmax_index = xx(lambda x: max(range(len(x)), key=x.__getitem__))  # 列表中最小和最大值的索引
xmin_index = xx(lambda x: min(range(len(x)), key=x.__getitem__))  # 列表中最小和最大值的索引
xmost_freq = xx(lambda x: max(set(x), key=x.count))  # 查找列表中频率最高的值, key作用于set(x), 可类推出其他用法

# print
xprint = xx(lambda obj, bg='blue': cprint(obj, bg))
xtqdm = xx(lambda iterable, desc=None: tqdm(iterable, desc))

# base types
xtuple, xlist, xset = xx(tuple), xx(list), xx(set)

# string
xjoin = xx(lambda string, sep=' ': sep.join(string))
xcut = xx(lambda string, cut_all=False: jieba.cut(string, cut_all=cut_all))

# list transform
xgroup_by_step = xx(lambda ls, step=3: [ls[idx: idx + step] for idx in range(0, len(ls), step)])


# dict
@xx
def xjson(dict_):
    _ = json.dumps(dict_, default=lambda obj: obj.__dict__, sort_keys=True, indent=4)
    return _


@xx
def xSeries(iterable, name='iterable'):
    if isinstance(iterable, pd.Series):
        return iterable
    else:
        return pd.Series(iterable, name=name)


@xx
def xDataframe(iterable, name='iterable'):
    if isinstance(iterable, pd.DataFrame):
        return iterable
    else:
        return pd.DataFrame({name: iterable})


# 高阶函数
xmap = xx(lambda iterable, func: map(func, iterable))
xreduce = xx(lambda iterable, func: reduce(func, iterable))
xfilter = xx(lambda iterable, func: filter(func, iterable))


# multiple
@xx
def xThreadPoolExecutor(iterable, func, max_workers=5):
    with ThreadPoolExecutor(max_workers) as pool:
        return pool.map(func, iterable)


@xx
def xProcessPoolExecutor(iterable, func, max_workers=5):
    with ProcessPoolExecutor(max_workers) as pool:
        return pool.map(func, iterable)


import jovian


def jupyter_commit(filename=None):
    print(
        "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJmcmVzaCI6ZmFsc2UsImlkZW50aXR5Ijp7InVzZXJuYW1lIjoiSmllLVl1YW4iLCJpZCI6Njd9LCJ0eXBlIjoiYWNjZXNzIiwiZXhwIjoxNTUyOTY1NTkzLCJpYXQiOjE1NTIzNjA3OTMsIm5iZiI6MTU1MjM2MDc5MywianRpIjoiNjM1ZTg2MjQtYjA1ZC00NGJmLTljYjAtOGVjOGRmM2ExNmJkIn0.5jglhEGGs12ITl-DWWaFL-BVPhCzaDEeMKIJvEI-bbA")
    print('\n')
    jovian.commit(filename=filename, env_type='pip')



# -*- coding: utf-8 -*-
"""
__title__ = 'vibora_test'
__author__ = 'JieYuan'
__mtime__ = '18-11-6'
"""
import jieba
import json
from vibora import Vibora, Request
from vibora.responses import JsonResponse
# from urllib import parse
# parse.unquote(parse.quote('中文'))

app = Vibora()


@app.route('/json', methods=['POST'])
async def home(request: Request):
    content = await request.stream.read()
    content = eval(content)
    content = jieba.lcut(content.get('text', ''))
    return JsonResponse(content)

# import requests
# r = requests.post('http://0.0.0.0:5000/json', json={'text': '小米公司还不错啊'})
# r.json()

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000, workers=4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'translate'
__author__ = 'JieYuan'
__mtime__ = '19-3-1'
"""
import requests

"""
https://www.cnblogs.com/fanyang1/p/9414088.html
"""


class Translater(object):

    def __init__(self):
        pass

    def get_zh(self, text='translater', api='google'):
        params = {'q': text}
        if api == 'google':
            url = "http://translate.google.cn/translate_a/single?client=gtx&dt=t&dj=1&ie=UTF-8&sl=auto&tl=zh"
            return requests.get(url, params).json()

    def get_en(self):
        pass



import numpy as np
from tqdm import tqdm

"""
1. 加和/平均
2. Concatenated p-mean Word Embeddings: https://blog.csdn.net/triplemeng/article/details/81298100
3. InferSent: https://github.com/facebookresearch/InferSent
"""


class Sent2Vec(object):

    def __init__(self, fname):
        self.embeddings = {}
        self.word_size = None
        self._load_wv(fname)

    def sent2vec(self, sentence, tokenizer=str.split, mode='sum', normalize=False):
        words = [w for w in tokenizer(sentence) if w in self.embeddings]

        if words:
            if mode == 'sum':
                _ = np.array([self.embeddings[w] for w in words]).sum(axis=0)
                return _ / np.sqrt(np.clip((_ ** 2).sum(), 1e-12, None)) if normalize else _
            elif mode == 'mean':
                _ = np.array([self.embeddings[w] for w in words]).mean(axis=0)
                return _ / np.sqrt(np.clip((_ ** 2).sum(), 1e-12, None)) if normalize else _
            else:
                return np.zeros(self.word_size)
        else:
            return np.zeros(self.word_size)

    def _load_wv(self, fname):
        with open(fname) as f:
            for line in tqdm(f, 'Load Vectors ...'):
                line = line.strip().split()
                if len(line) > 2:
                    self.embeddings[line[0]] = np.asarray(line[1:], dtype='float32')
            self.word_size = len(line[1:])



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'word_cloud'
__author__ = 'JieYuan'
__mtime__ = '19-3-1'
"""

# from IPython.display import Image
# import numpy as np
# from wordcloud import WordCloud, ImageColorGenerator
# cloud_mask = np.array(Image.open("/home/yuanjie/desktop/xiaomi.jpg"))
#
# #生成wordcloud对象
# wc = WordCloud(background_color="white",
#     mask=cloud_mask,
#     max_words=200,
#     font_path="/home/yuanjie/desktop/simhei/simhei.ttf",
#     min_font_size=15,
#     max_font_size=50,
#     width=400)
#
# wc.generate_from_frequencies(tf)
# wc.to_file("pic.png")
# Image.open('./pic.png')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'setup.py'
__author__ = 'JieYuan'
__mtime__ = '18-12-14'
"""
import re
import os
from setuptools import find_packages, setup

with open('yuan/__init__.py', encoding='utf8') as f:
    version = re.search(r'__version__ = \'(.*?)\'', f.read()).group(1)

with open("README.md", encoding='utf-8') as f:
    long_description = f.read()


def get_requirements():
    _ = './requirements.txt'
    if os.path.isfile(_):
        with open(_, encoding='utf-8') as f:
            return f.read().split()


setup(
    name='Yuan',
    version=version,
    url='https://github.com/Jie-Yuan/Yuan',
    keywords=["tool wheel", "yuanjie", 'utils', '轮子'],
    description=('description'),
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='JieYuan',
    author_email='313303303@qq.com',
    maintainer='JieYuan',
    maintainer_email='313303303@qq.com',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    package_data={'': ['*.*']},
    platforms=["all"],
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries'
    ],

    install_requires=get_requirements()
)
